package SAE.ApiREST.WebService.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import SAE.ApiREST.WebService.Response;
import SAE.ApiREST.WebService.exception.ArticleException;
import SAE.ApiREST.WebService.model.Article;
import SAE.ApiREST.WebService.service.IArticleService;

import java.util.ArrayList;
import java.util.List;

@Controller
@RequestMapping("/ArticleWebService")
public class ArticleControler {
    @Autowired
    IArticleService articleService;

    // region POST
    @PostMapping(
        value = "/addArticle",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE
        )
    public @ResponseBody Response addArticle(
        @RequestBody Article article
    ) {
        return articleService.addArticle(article);
    }
    // endregion

    // region PUT
    @PutMapping(
        value = "/updateTitle/{title}",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Article updateTitle(
        @PathVariable("title") String title,
        @RequestBody Article article
    ) {
        return articleService.updateTitle(article, title);
    }
    
    @PutMapping(
        value = "/updateUrl/{url}",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Article updateUrl(
        @PathVariable("url") String url,
        @RequestBody Article article
    ) {
        return articleService.updateUrl(article, url);
    }
    
    @PutMapping(
        value = "/updateDatePublished/{datePublished}",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Article updateDatePublished(
        @PathVariable("datePublished") String datePublished,
        @RequestBody Article article
    ) {
        return articleService.updateDatePublished(article, datePublished);
    }
    
    @PutMapping(
        value = "/updateDateAdded/{dateAdded}",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Article updateDateAdded(
        @PathVariable("dateAdded") String dateAdded,
        @RequestBody Article article
    ) {
        return articleService.updateDateAdded(article, dateAdded);
    }
    
    @PutMapping(
        value = "/changeVisibility",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Article changeVisibility(
        @RequestBody Article article
    ) {
        return articleService.changeVisibility(article);
    }
    
    @PutMapping(
        value = "/updateType/{type}",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Article updateType(
        @PathVariable("type") Integer type,
        @RequestBody Article article
    ) {
        return articleService.updateType(article, type);
    }
    // endregion

    // region GET
    @GetMapping(value = "/getAllArticle", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getAllArticles() {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getAllArticles();
        
        if(results.isEmpty()) {
            throw new ArticleException("No articles available");
        }
        
        return results;
    }

    @GetMapping(value = "/getArticleById/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody Article getArticlesById(@PathVariable(value = "id") Integer id) {
        Article results = articleService.getArticlesById(id);
        
        if(results == null) {
            throw new ArticleException("Undefined id");
        }
        
        return results;
    }

    @GetMapping(value = "/getArticlesByTitle/{title}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesByTitle(@PathVariable(value = "title") String title) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesByTitle(title);
        
        if(results.isEmpty()) {
            throw new ArticleException("Undefined title");
        }
        
        return results;
    }

    @GetMapping(value = "/getArticlesByType/{type}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesByType(@PathVariable(value = "type") Integer type) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesByType(type);

        if (results.isEmpty()) {
            throw new ArticleException(String.format("No content of type %d", type));
        }

        return results;
    }

    @GetMapping(value = "/getVisibleArticles", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getVisibleArticles() {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getVisibleArticles();

        if (results.isEmpty()) {
            throw new ArticleException("No visible article");
        }

        return results;
    }

    @GetMapping(value = "/getInvisibleArticles", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getInvisibleArticles() {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getInvisibleArticles();

        if (results.isEmpty()) {
            throw new ArticleException("No invisible article");
        }

        return results;
    }

    @GetMapping(value = "/getArticlesAddedBefore/{dateAdded}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesAddedBefore(@PathVariable(value = "dateAdded") String dateAdded) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesAddedBefore(dateAdded);

        if (results.isEmpty()) {
            throw new ArticleException(String.format("No article added before %t", dateAdded));
        }
        return results;
    }

    @GetMapping(value = "/getArticlesAddedAfter/{dateAdded}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesAddedAfter(@PathVariable(value = "dateAdded") String dateAdded) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesAddedAfter(dateAdded);

        if (results.isEmpty()) {
            throw new ArticleException(String.format("No article added after %t", dateAdded));
        }
        return results;
    }

    @GetMapping(value = "/getArticlesAddedBetween/{beginning}/{end}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesAddedAfter(@PathVariable(value = "beginning") String beginning, @PathVariable(value = "end") String end) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesAddedBetween(beginning, end);

        if (results.isEmpty()) {
            throw new ArticleException(String.format("No article added between %t and %t", beginning, end));
        }
        return results;
    }

    @GetMapping(value = "/getArticlesPublishedBefore/{datePublished}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesPublishedBefore(@PathVariable(value = "datePublished") String datePublished) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesPublishedBefore(datePublished);

        if (results.isEmpty()) {
            throw new ArticleException(String.format("No article published before %t", datePublished));
        }

        return results;
    }

    @GetMapping(value = "/getArticlesPublishedAfter/{datePublished}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesPublishedAfter(@PathVariable(value = "datePublished") String datePublished) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesPublishedAfter(datePublished);

        if (results.isEmpty()) {
            throw new ArticleException(String.format("No article published after %t", datePublished));
        }

        return results;
    }

    @GetMapping(value = "/getArticlesPublishedBetween/{beginning}/{end}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Article> getArticlesPublishedAfter(@PathVariable(value = "beginning") String beginning, @PathVariable(value = "end") String end) {
        ArrayList<Article> results = (ArrayList<Article>) articleService.getArticlesPublishedBetween(beginning, end);

        if (results.isEmpty()) {
            throw new ArticleException(String.format("No article published between %t and %t", beginning, end));
        }

        return results;
    }
    
    // endregion

    // region DELETE
    @DeleteMapping(
        value = "/deleteArticleFromId/{id}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticleFromId(
        @PathVariable("id") Integer id
    ) {
        return articleService.deleteArticleFromId(id);
    }

    @DeleteMapping(
        value = "/deleteArticleFromTitle/{title}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticleFromTitle(
        @PathVariable("title") String title
    ) {
        return articleService.deleteArticleFromTitle(title);
    }

    @DeleteMapping(
        value = "/deleteArticleFromUrl/{url}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticleFromUrl(
        @PathVariable("url") String url
    ) {
        return articleService.deleteArticleFromUrl(url);
    }

    @DeleteMapping(
        value = "/deleteArticleAddedBefore/{date}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticleAddedBefore(
        @PathVariable("date") String date
    ) {
        return articleService.deleteArticleAddedBefore(date);
    }

    @DeleteMapping(
        value = "/deleteArticleAddedAfter/{date}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticleAddedAfter(
        @PathVariable("date") String date
    ) {
        return articleService.deleteArticleAddedAfter(date);
    }@DeleteMapping(
        value = "/deleteArticleAddedBetween/{beginning}/{end}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticleAddedBetween(
        @PathVariable("beginning") String beginning,
        @PathVariable("end") String end
    ) {
        return articleService.deleteArticleAddedBetween(beginning, end);
    }

    @DeleteMapping(
        value = "/deleteArticlePublishedBefore/{date}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticlePublishedBefore(
        @PathVariable("date") String date
    ) {
        return articleService.deleteArticlePublishedBefore(date);
    }

    @DeleteMapping(
        value = "/deleteArticlePublishedAfter/{date}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticlePublishedAfter(
        @PathVariable("date") String date
    ) {
        return articleService.deleteArticlePublishedAfter(date);
    }
    
    @DeleteMapping(
        value = "/deleteArticlePublishedBetween/{beginning}/{end}",
        produces = MediaType.APPLICATION_JSON_VALUE
    )
    public @ResponseBody Response deleteArticlePublishedBetween(
        @PathVariable("beginning") String beginning,
        @PathVariable("end") String end
    ) {
        return articleService.deleteArticlePublishedBetween(beginning, end);
    }
    // endregion
}
